package com.viapidemo.androiddemo;

import android.app.Activity;
import android.os.Bundle;
import android.os.Looper;
import android.text.TextUtils;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.Nullable;

// 1、这里只是以ocr下的RecognizeBankCard能力为例，其他能力请引入相应类目的包和相关类
import com.aliyun.ocr20191230.Client;
import com.aliyun.ocr20191230.models.RecognizeBankCardResponse;
import com.aliyun.tea.TeaModel;
import com.aliyun.teaopenapi.models.Config;

import java.io.InputStream;
import java.net.URL;

public class MainActivity extends Activity {

    private String TAG = "AndroidDemo";

    /*
      这个client是为了请求服务端接口，这里只是为了端上演示，所以将代码写在了Android端
      真正上线不建议将ACCESS_KEY_ID和ACCESS_KEY_SECRET写在端上，会有泄漏风险，建议将请求服务端接口代码写到您的服务端
     */
    private Client client = null;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        // 让TextView能滑动
        ((TextView)findViewById(R.id.hintText)).setMovementMethod(ScrollingMovementMethod.getInstance());
        try {
            // 2、"YOUR_ACCESS_KEY_ID", "YOUR_ACCESS_KEY_SECRET" 的生成请参考https://help.aliyun.com/document_detail/175144.html
            // 如果您是用的子账号AccessKey，还需要为子账号授予权限AliyunVIAPIFullAccess，请参考https://help.aliyun.com/document_detail/145025.html
            Config config = new Config()
                    // 您的 AccessKey ID
                    .setAccessKeyId("YOUR_ACCESS_KEY_ID")
                    // 您的 AccessKey Secret
                    .setAccessKeySecret("YOUR_ACCESS_KEY_SECRET");
            // 3、访问的域名。注意：这个地方需要求改为相应类目的域名，参考：https://help.aliyun.com/document_detail/143103.html
            config.endpoint = "ocr.cn-shanghai.aliyuncs.com";
            client = new Client(config);
        } catch (Exception e) {
            Log.e(TAG, String.format("onCreate: %s", e.getMessage()));
            showToastMsg("初始化失败");
        }
    }

    /**
     * 文件在同地域OSS，调用API，以RecognizeBankCard为例。
     * @param view
     */
    public void callApiShanghaiOss(View view) {

        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    Log.d(TAG, String.format("begin callApi: %s %s", "RecognizeBankCard", "http://viapi-test.oss-cn-shanghai.aliyuncs.com/viapi-3.0domepic/ocr/RecognizeBankCard/yhk1.jpg"));
                    // 4、这里只是以ocr下的RecognizeBankCard为例，其他能力请使用相应类目的包和类，具体入参设置需要参考具体能力的文档
                    com.aliyun.ocr20191230.models.RecognizeBankCardRequest recognizeBankCardRequest = new com.aliyun.ocr20191230.models.RecognizeBankCardRequest()
                            .setImageURL("http://viapi-test.oss-cn-shanghai.aliyuncs.com/viapi-3.0domepic/ocr/RecognizeBankCard/yhk1.jpg");
                    com.aliyun.teautil.models.RuntimeOptions runtime = new com.aliyun.teautil.models.RuntimeOptions();
                    // 5、这里只是以ocr下的RecognizeBankCard为例，其他能力请使用相应类目的包和类，注意，recognizeBankCardWithOptions方法名也需要改成对应能力的方法名
                    RecognizeBankCardResponse resp = client.recognizeBankCardWithOptions(recognizeBankCardRequest, runtime);
                    // 获取整体结果。部分能力会输出url链接，通过toJSONString转换后可能有编码问题，但是通过单个字段获取是没问题的。
                    Log.d(TAG, String.format("%s", com.aliyun.teautil.Common.toJSONString(TeaModel.buildMap(resp))));
                    // 获取单个字段，这里只是一个例子，具体能力下的字段需要看具体能力的文档
                    Log.d(TAG, String.format("%s", resp.getBody().getData().getCardNumber()));
                    showToastMsg("调用成功，银行卡号：" + resp.getBody().getData().getCardNumber());
                } catch (com.aliyun.tea.TeaException teaException) {
                    Log.d(TAG, "teaException.getCode(): " + teaException.getCode());
                    if("InvalidAccessKeyId.NotFound".equals(teaException.getCode())) {
                        showToastMsg("请求报错，请检查您代码中的YOUR_ACCESS_KEY_ID和YOUR_ACCESS_KEY_SECRET是否已经修改正确。");
                    } else if("InvalidApi.NotPurchase".equals(teaException.getCode())) {
                        showToastMsg("请求报错，您的账号未开通视觉智能开放平台相应类目，请进行开通：https://help.aliyun.com/document_detail/465341.html");
                    } else if("Unauthorized".equals(teaException.getCode())) {
                        showToastMsg("请求报错，您的子账号未授予AliyunVIAPIFullAccess权限，请参考https://help.aliyun.com/document_detail/145025.html");
                    } else if("InvalidAction.NotFound".equals(teaException.getCode())) {
                        showToastMsg("请求报错，请检查您调用的API和类目是否匹配，API和类目的关系请参考：https://help.aliyun.com/document_detail/465341.html，和访问的域名是否匹配，类目和域名的关系请参考：https://help.aliyun.com/document_detail/143103.html");
                    } else {
                        showToastMsg(teaException.getMessage());
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    Log.e(TAG, e.getMessage());
                }
            }
        }).start();

    }


    /**
     * 文件在本地，调用API，以RecognizeBankCard为例。
     * @param view
     */
    public void callApiLocal(View view) {

        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    // filePath请改成您的真实文件路径
                    String filePath = "resource/test_images/yhk1.jpg";
                    Log.d(TAG, String.format("begin callApi: %s %s", "RecognizeBankCard", filePath));
                    // 使用文件，文件通过inputStream传入接口。这里只是演示了assets下的文件如何转为stream，如果文件来自其他地方，如sdcard或者摄像头，请自行查看android开发文档或教程将文件转为stream之后传入。
                    InputStream inputStream = MainActivity.this.getAssets().open(filePath);
                    // 4、这里只是以ocr下的RecognizeBankCard为例，其他能力请使用相应类目的包和类，具体入参设置需要参考具体能力的文档
                    com.aliyun.ocr20191230.models.RecognizeBankCardAdvanceRequest recognizeBankCardAdvanceRequest = new com.aliyun.ocr20191230.models.RecognizeBankCardAdvanceRequest()
                            .setImageURLObject(inputStream);
                    com.aliyun.teautil.models.RuntimeOptions runtime = new com.aliyun.teautil.models.RuntimeOptions();
                    // 5、这里只是以ocr下的RecognizeBankCard为例，其他能力请使用相应类目的包和类，注意，recognizeBankCardAdvance方法名也需要改成对应能力的方法名
                    RecognizeBankCardResponse resp = client.recognizeBankCardAdvance(recognizeBankCardAdvanceRequest, runtime);
                    // 获取整体结果。部分能力会输出url链接，通过toJSONString转换后可能有编码问题，但是通过单个字段获取是没问题的。
                    Log.d(TAG, String.format("%s", com.aliyun.teautil.Common.toJSONString(TeaModel.buildMap(resp))));
                    // 获取单个字段，这里只是一个例子，具体能力下的字段需要看具体能力的文档
                    Log.d(TAG, String.format("%s", resp.getBody().getData().getCardNumber()));
                    showToastMsg("调用成功，银行卡号：" + resp.getBody().getData().getCardNumber());
                } catch (com.aliyun.tea.TeaException teaException) {
                    Log.d(TAG, "teaException.getCode(): " + teaException.getCode());
                    if("InvalidAccessKeyId.NotFound".equals(teaException.getCode())) {
                        showToastMsg("请求报错，请检查您代码中的YOUR_ACCESS_KEY_ID和YOUR_ACCESS_KEY_SECRET是否已经修改正确。");
                    } else if("InvalidApi.NotPurchase".equals(teaException.getCode())) {
                        showToastMsg("请求报错，您的账号未开通视觉智能开放平台相应类目，请进行开通：https://help.aliyun.com/document_detail/465341.html");
                    } else if("Unauthorized".equals(teaException.getCode())) {
                        showToastMsg("请求报错，您的子账号未授予AliyunVIAPIFullAccess权限，请参考https://help.aliyun.com/document_detail/145025.html");
                    } else if("InvalidAction.NotFound".equals(teaException.getCode())) {
                        showToastMsg("请求报错，请检查您调用的API和类目是否匹配，API和类目的关系请参考：https://help.aliyun.com/document_detail/465341.html，和访问的域名是否匹配，类目和域名的关系请参考：https://help.aliyun.com/document_detail/143103.html");
                    } else {
                        showToastMsg(teaException.getMessage());
                    }
                } catch (java.io.FileNotFoundException e) {
                    showToastMsg("文件找不到，请检查代码中的filePath路径是否正确");
                    Log.e(TAG, e.getMessage());
                } catch (Exception e) {
                    e.printStackTrace();
                    Log.e(TAG, e.getMessage());
                }
            }
        }).start();

    }

    /**
     * 文件在任意URL，调用API，以RecognizeBankCard为例。
     * @param view
     */
    public void callApiAnyUrl(View view) {

        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    Log.d(TAG, String.format("begin callApi: %s %s", "RecognizeBankCard", "https://viapi-test-bj.oss-cn-beijing.aliyuncs.com/viapi-3.0domepic/ocr/RecognizeBankCard/yhk1.jpg"));
                    // 使用任意URL
                    URL url = new URL("https://viapi-test-bj.oss-cn-beijing.aliyuncs.com/viapi-3.0domepic/ocr/RecognizeBankCard/yhk1.jpg");
                    InputStream inputStream = url.openConnection().getInputStream();
                    // 4、这里只是以ocr下的RecognizeBankCard为例，其他能力请使用相应类目的包和类，具体入参设置需要参考具体能力的文档
                    com.aliyun.ocr20191230.models.RecognizeBankCardAdvanceRequest recognizeBankCardAdvanceRequest = new com.aliyun.ocr20191230.models.RecognizeBankCardAdvanceRequest()
                            .setImageURLObject(inputStream);
                    com.aliyun.teautil.models.RuntimeOptions runtime = new com.aliyun.teautil.models.RuntimeOptions();
                    // 5、这里只是以ocr下的RecognizeBankCard为例，其他能力请使用相应类目的包和类，注意，recognizeBankCardAdvance方法名也需要改成对应能力的方法名
                    RecognizeBankCardResponse resp = client.recognizeBankCardAdvance(recognizeBankCardAdvanceRequest, runtime);
                    // 获取整体结果。部分能力会输出url链接，通过toJSONString转换后可能有编码问题，但是通过单个字段获取是没问题的。
                    Log.d(TAG, String.format("%s", com.aliyun.teautil.Common.toJSONString(TeaModel.buildMap(resp))));
                    // 获取单个字段，这里只是一个例子，具体能力下的字段需要看具体能力的文档
                    Log.d(TAG, String.format("%s", resp.getBody().getData().getCardNumber()));
                    showToastMsg("调用成功，银行卡号：" + resp.getBody().getData().getCardNumber());
                } catch (com.aliyun.tea.TeaException teaException) {
                    Log.d(TAG, "teaException.getCode(): " + teaException.getCode());
                    if("InvalidAccessKeyId.NotFound".equals(teaException.getCode())) {
                        showToastMsg("请求报错，请检查您代码中的YOUR_ACCESS_KEY_ID和YOUR_ACCESS_KEY_SECRET是否已经修改正确。");
                    } else if("InvalidApi.NotPurchase".equals(teaException.getCode())) {
                        showToastMsg("请求报错，您的账号未开通视觉智能开放平台相应类目，请进行开通：https://help.aliyun.com/document_detail/465341.html");
                    } else if("Unauthorized".equals(teaException.getCode())) {
                        showToastMsg("请求报错，您的子账号未授予AliyunVIAPIFullAccess权限，请参考https://help.aliyun.com/document_detail/145025.html");
                    } else if("InvalidAction.NotFound".equals(teaException.getCode())) {
                        showToastMsg("请求报错，请检查您调用的API和类目是否匹配，API和类目的关系请参考：https://help.aliyun.com/document_detail/465341.html，和访问的域名是否匹配，类目和域名的关系请参考：https://help.aliyun.com/document_detail/143103.html");
                    } else {
                        showToastMsg(teaException.getMessage());
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    Log.e(TAG, e.getMessage());
                }
            }
        }).start();

    }

    /**
     * 显示toast
     * @param msg 内容
     */
    private void showToastMsg(final String msg) {
        if (TextUtils.isEmpty(msg)) {
            return;
        }

        if (Looper.getMainLooper() == Looper.myLooper()) {
            Toast.makeText(MainActivity.this, msg, Toast.LENGTH_SHORT).show();
        } else {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    Toast.makeText(MainActivity.this, msg, Toast.LENGTH_SHORT).show();
                }
            });
        }
    }

}
